unit DtFunc97;

(******************************************************************************
Misc. functions for TDateEdit97

Author name=BOURMAD Mehdi
Author E-mail=bourmad@multimania.com
Author URL=www.multimania.com/bourmad
******************************************************************************)

interface

uses
  WinTypes, WinProcs, Consts, Classes, Dialogs, DsgnIntf, Graphics, SysUtils,
  DtMisc97;

procedure InfoDebug (aStr : String);
procedure InfoDate (aDate : tDateTime);

function IsLeapYear (aDate: tDateTime) : Boolean;
function DaysInMonth (aDate: tDateTime): Integer;
function GetFirstOfMonth (aDate: tDateTime): tDateTime;
{$IFDEF VER80} {replace delphi 1 function StrToDate by this one }
function D1StrToDate (const S: string): TDateTime;
{$ENDIF}
function TextToDate (aStr: String) : tDateTime;

implementation

{********************** Info Debug ************************
 *****  Display a text (string)                       *****
 *****  in standard debug window.                     *****
 **********************************************************}
procedure InfoDebug (aStr : String);
var
  Buf : Array[0..255] of char;
begin
  StrPCopy (Buf, aStr+#13#10);
  OutputDebugString (Buf);
end;

{********************** Info Date *************************
 *****  Display a date                                *****
 *****  in standard debug window.                     *****
 **********************************************************}
procedure InfoDate (aDate : tDateTime);
var
  jj, mm, aa : word;
begin
  DecodeDate (aDate, aa, mm, jj);
  InfoDebug (IntToStr(jj)+DateSeparator
       +IntToStr(mm)+DateSeparator
       +IntToStr(aa));
end;

{********************** Is Leap Year **********************
 *****  This function returns true if current year    *****
 *****  is a leap year.                               *****
 **********************************************************}
function IsLeapYear (aDate: tDateTime) : Boolean;
var
  aYear, aMonth, aDay : word;
begin
  DecodeDate (aDate, aYear, aMonth, aDay);
  Result := (aYear mod 4 = 0) and
           ((aYear mod 100 <> 0) or (aYear mod 400 = 0));
end;

{********************** Days In Month *********************
 *****  This function returns the number of days in   *****
 *****  the month specified in nMonth.                *****
 **********************************************************}
function DaysInMonth(aDate: tDateTime): Integer;
var
  aYear, aMonth, aDay : word;
begin
  DecodeDate (aDate, aYear, aMonth, aDay);
  Result := DaysPerMonth[aMonth];
  if (aMonth = 2) and IsLeapYear(aDate) then Inc(Result); { leap-year Feb is special}
end;

{****************** Get First Of Month ********************
 *****  This function returns the first day           *****
 *****  of the current month.                         *****
 **********************************************************}
function GetFirstOfMonth (aDate: tDateTime): tDateTime;
var
  aYear, aMonth, aDay : word;
begin
  DecodeDate (aDate, aYear, aMonth, aDay);
  Result := EncodeDate (aYear, aMonth, 1);
end;

{$IFDEF VER80} {replace delphi 1 function StrToDate by this one }
{********************* D1StrToDate ***********************
 *****  Convert string to date                        *****
 *****  Correct Delphi 1 pb when year use 2 digits    *****
 *****  Now, use current century instead of 1900      *****
 **********************************************************}
type
  TDateOrder = (doMDY, doDMY, doYMD);

function GetDateOrder(const DateFormat: string): TDateOrder;
var
  I: Integer;
begin
  I := 1;
  while I <= Length(DateFormat) do
  begin
    case Chr(Ord(DateFormat[I]) and $DF) of
      'Y': Result := doYMD;
      'M': Result := doMDY;
      'D': Result := doDMY;
    else
      Inc(I);
      Continue;
    end;
    Exit;
  end;
  Result := doMDY;
end;

procedure ScanBlanks(const S: string; var Pos: Integer);
var
  I: Integer;
begin
  I := Pos;
  while (I <= Length(S)) and (S[I] = ' ') do Inc(I);
  Pos := I;
end;

function ScanNumber(const S: string; var Pos: Integer;
  var Number: Word): Boolean;
var
  I: Integer;
  N: Word;
begin
  Result := False;
  ScanBlanks(S, Pos);
  I := Pos;
  N := 0;
  while (I <= Length(S)) and (S[I] in ['0'..'9']) and (N < 1000) do
  begin
    N := N * 10 + (Ord(S[I]) - Ord('0'));
    Inc(I);
  end;
  if I > Pos then
  begin
    Pos := I;
    Number := N;
    Result := True;
  end;
end;

function ScanChar(const S: string; var Pos: Integer; Ch: Char): Boolean;
begin
  Result := False;
  ScanBlanks(S, Pos);
  if (Pos <= Length(S)) and (S[Pos] = Ch) then
  begin
    Inc(Pos);
    Result := True;
  end;
end;

function CurrentYear: Word; assembler;
asm
        MOV     AH,2AH
        INT     21H
        MOV     AX,CX
end;

function ScanDate(const S: string; var Pos: Integer;
  var Date: TDateTime): Boolean;
var
  DateOrder: TDateOrder;
  I: Integer;
  N1, N2, N3, Y, M, D: Word;
begin
  Result := False;
  DateOrder := GetDateOrder(ShortDateFormat);
  if not (ScanNumber(S, Pos, N1) and ScanChar(S, Pos, DateSeparator) and
    ScanNumber(S, Pos, N2)) then Exit;
  if ScanChar(S, Pos, DateSeparator) then
  begin
    if not ScanNumber(S, Pos, N3) then Exit;
    case DateOrder of
      doMDY: begin Y := N3; M := N1; D := N2; end;
      doDMY: begin Y := N3; M := N2; D := N1; end;
      doYMD: begin Y := N1; M := N2; D := N3; end;
    end;
    if Y <= 99 then Inc(Y, CurrentYear div 100 * 100);
  end else
  begin
    Y := CurrentYear;
    if DateOrder = doDMY then
    begin
      D := N1; M := N2;
    end else
    begin
      M := N1; D := N2;
    end;
  end;
  ScanBlanks(S, Pos);
  try
    Result := true;
    Date := EncodeDate(Y, M, D);
  except
    Result := false;
  end;
end;

function D1StrToDate(const S: string): TDateTime;
var
  Pos: Integer;
begin
  Pos := 1;
  if not ScanDate(S, Pos, Result) or (Pos <= Length(S)) then
    raise EConvertError.CreateResFmt(SInvalidDate, [S]);
end;
{$ENDIF}

{*********************- TextToDate ************************
 *****  Convert string to date                        *****
 *****  Correct Delphi 1 pb when year use 2 digits    *****
 *****  Now, use current century instead of 1900      *****
 **********************************************************}
function TextToDate (aStr: String) : tDateTime;
begin
  {$IFDEF VER80}
  Result := D1StrToDate (aStr);
  {$ELSE}
  Result := StrToDate (aStr);
  {$ENDIF}
end;

end.
